#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#  System-config-pam account access interface
#
#  This file contains some methods to get account access
#  information from Linux-PAM
#
#  Author: MengYuan <mengyuan@kylinos.cn>
#
#  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the
#  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
#  Boston, MA 02110-1301, USA.

access_conf = { "nullok":0,
                "control":0,
                "deny":0,
                "entry":[]
    }

CONTROL = { None:0, "login":1, "lightdm":2, "sshd":3, "login lightdm":4,
            "login sshd":5, "lightdm sshd":6, "login lightdm sshd":7}

COM_AUTH = "/etc/pam.d/common-auth"
CONF_FILE = ["/etc/pam.d/login", "/etc/pam.d/lightdm", "/etc/pam.d/sshd"]
ACCESS_CONF =  "/etc/security/access.conf"

def get_access_conf():
    ret = access_conf

    ret["nullok"] = read_com_auth()
    ret["control"] = get_control()
    ret["entry"] = get_access_entry()
    if ["-","ALL","ALL"] in ret["entry"]:
        ret["deny"] = 1

    return ret

def read_com_auth():
    try :
        f = open(COM_AUTH,"r")
    except IOError as e:
        print("read common-auth failed",e) 
        return
    else:
        ret = 0
        tmp = f.readlines()
        f.close()
        for i in tmp:
            if i.strip().startswith("#") or ret == 1:
                continue
            if "unix.so" in i and "nullok" in i:
                ret = 1
    return ret

def get_control():
    ret = None
    for i in CONF_FILE:
        try:
            f = open(i,"r")
        except IOError as e:
            print("Cannot read account access file %s" %i, e)
        else:
            for x in f.readlines():
                if x.strip().startswith("#") or ret == 1:
                    continue 
                if "pam_access.so" in x and "echo.so" not in x:
                    if ret == None:
                        ret = i.split("/")[-1]
                    else:
                        ret = ret + " " + i.split("/")[-1]
            f.close()
    if ret in CONTROL:
        return CONTROL[ret]
    else:
        return 0

def get_access_entry():
    try:
        f = open(ACCESS_CONF, "r")
    except IOError as e:    
        print("access.conf not exists",e)
        return []
    else:
        t = f.readlines()
        f.close()
        entry = []
        for i in t:
            m = []
            if i.startswith("+") or i.startswith("-"):
                for x in i.split(":"):
                    m.append(x.strip())
                entry.append(m)
        if ['-', 'ALL', 'ALL'] in entry:
            return entry[0:entry.index(['-', 'ALL', 'ALL'])]
        else:
            return entry

        return entry
