#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#  System-config-pam account lock settings interface
#
#  This file provides some methods to get account lock information
#  and set account lock
#
#  Author: MengYuan <mengyuan@kylinos.cn>
#
#  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the
#  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
#  Boston, MA 02110-1301, USA.

import configparser
import shutil
import os
from util import handle_int_option

tally_option = {"tally" : 0,
	        	"unlock_time" : 0,
	        	"deny" : 0,
		        "deny_root" : 0,
		        "root_unlock_time" : 0}
COM_AUTH = "/etc/pam.d/common-auth"
SAVEDIR = "/usr/share/system-config-pam/"
CONFDIR = "/usr/share/pam-configs/"
unixfile = CONFDIR + "unix"
lastime = SAVEDIR + ".config"

def get_last_lock():
    ret = tally_option
    cf = configparser.ConfigParser()
    cf.read(lastime)

    ret["unlock_time"] = cf["time"]["unlock_time"]
    ret["root_unlock_time"] = cf["time"]["root_unlock_time"]
    ret["deny"] = cf["time"]["deny"]

    return ret

def get_account_lock():
    ret = get_last_lock()

    try:
        fp = open(COM_AUTH,"r")
    except IOError as e:
        print("Cannot open common-auth",e)
        return
    else:    
        tmp = fp.readlines()
        fp.close()

    t = []
    for i in tmp:
        if i.strip().startswith("#") or ("unix" not in i):
            continue
        else:
            t.append(i.strip())

    if t != []:
        x = t[0].split()
        for i in x:
            if i.startswith("deny"):
                try:
                    ret["deny"] = int(i.split("=")[1])%65536
                except ValueError as e:
                    print("Get Invalid Options",e)
                    ret["deny"] = 0
                if ret["deny"] > 0:
                    ret["tally"] = 1
            if i.startswith("unlock_time"):
                ret["unlock_time"] = handle_int_option(i.split("=")[1])
            if i.startswith("root_unlock"):
                ret["deny_root"] = 1
                ret["root_unlock_time"] = handle_int_option(i.split("=")[1])
    return ret

def set_unix_option(option):
    if option["tally"] == 0 or option["deny"] == 0:
        t = "\n"
        cf = configparser.ConfigParser()
        cf.read(lastime)
        cf["time"]["unlock_time"] = str(option["unlock_time"])
        cf["time"]["root_unlock_time"] = str(option["root_unlock_time"])
        cf["time"]["deny"] = str(option["deny"])
        with open(lastime, "w") as configfile:
            cf.write(configfile)
    else:
        if option["deny_root"] == 0:
            t = " deny=" + str(option["deny"]) +\
                " unlock_time=" + str(option["unlock_time"]) + "\n"
        else:
            t = " deny=" + str(option["deny"]) +\
                " unlock_time=" + str(option["unlock_time"]) + \
                " root_unlock_time=" + str(option["root_unlock_time"]) + "\n"

    cmd = "cp " + SAVEDIR + "unix " + unixfile
    c = os.system(cmd)
    if c != 0:
        print("Cannot initial tally2")
        raise RuntimeError

    try:
        f = open(unixfile,"r")
    except IOError as e:
        print("Cannot read unix file")
        raise e
    else:
        tmp = f.readlines()
        f.close()

    for i in tmp:
        if "Auth-Initial:" in i:
            c = tmp.index(i)
            break
    tmp[c + 1] = tmp[c + 1].rstrip() + t
    tmp[c - 1] = tmp[c - 1].rstrip() + t

    try:
        f = open(unixfile,"w")
    except IOError as e:
        print("Cannot write tally2 file")
        raise e
    else:
        f.writelines(tmp)
        f.close()

    try:
        os.system("pam-auth-update --package --force")
    except OSError as e:
        print("Cannot set tally2")
        raise e

